#!/usr/local/bin/perl
#
#  AdminMove.pl
#
#  Perl Script to move messages from one queue
#  to another, saving all message context.
#  Messages are moved under syncpoint.
#
#  Program will translate all queue and QMgr names
#  to uppercase.
#
#  Be sure to pass in the QMgr name if connecting
#  as a client.

use strict;
use MQSeries;

use vars qw(
    $HConn
    $CompCode
    $Reason
    $QMgrName
    $MsgCnt
    );

my $BufferLength = 4096;  ## 4k input buffer.  Change as necessary.

# ------------------------------------------------------------------
# Collect command-line args
# ------------------------------------------------------------------
my $NumARGS = @ARGV;
if ($NumARGS < 2 || $NumARGS > 3) {
    die "Incorrect number of arguments.\n$0 <source-queue> <target-queue> <optional QMgr>\n";
} 

my $SourceQueue = uc($ARGV[0]);
my $TargetQueue = uc($ARGV[1]);
if ( $NumARGS == 3 ) {
    $QMgrName   = uc($ARGV[2]);
}else{
    $QMgrName   = "";
}

# ------------------------------------------------------------------
# Enclose everything but the Close/Disconnect in a block
# If errors, "last" takes us to Close/Disconnect
# ------------------------------------------------------------------
{
    # ------------------------------------------------------------------
    # Connect 
    # ------------------------------------------------------------------
    print STDERR "\n";
    $HConn = MQCONN($QMgrName, $CompCode, $Reason);
    if ($CompCode == 0){
	print STDERR "Connected to ", $QMgrName || "Default QMgr", "\n";}
    else {
	print STDERR "Problem Connecting to Queue Manager $QMgrName \n";
	print STDERR "$Reason: ", MQReasonToText($Reason), "\n";
	last;
    }

    # ------------------------------------------------------------------
    # Open Input and Output Queues
    # ------------------------------------------------------------------
    my $OpenOptionsIP = MQOO_INPUT_AS_Q_DEF | MQOO_FAIL_IF_QUIESCING | MQOO_SAVE_ALL_CONTEXT;
    my $ObjDescIP = {
	    ObjectName        => $SourceQueue,
	};			

    my $HobjIP = MQOPEN($HConn, $ObjDescIP, $OpenOptionsIP, $CompCode, $Reason);
    if ($CompCode == 0){
	print STDERR "Connected to Input Queue: ",$ObjDescIP->{ObjectName}, "\n";}
    else {
	print STDERR "Problem Connecting to Queue: ",$ObjDescIP->{ObjectName}, "\n";
	print STDERR "$Reason: ", MQReasonToText($Reason), "\n";
	last;
    }


    my $OpenOptionsOP = MQOO_OUTPUT | MQOO_FAIL_IF_QUIESCING | MQOO_PASS_ALL_CONTEXT;
    my $ObjDescOP = {
	    ObjectName        => $TargetQueue,
	};

    my $HobjOP = MQOPEN($HConn, $ObjDescOP, $OpenOptionsOP, $CompCode, $Reason);
    if ($CompCode == 0) {
	print STDERR "Connected to Output Queue: ", $ObjDescOP->{ObjectName}, "\n";}
    else{
	print STDERR "Problem Opening Queue Object: ", $ObjDescOP->{ObjectName}, "\n";
	print STDERR "$Reason: ", MQReasonToText($Reason), "\n";
	last;
    }
    print STDERR "\n";

    {
	# ------------------------------------------------------------------
	# Loop through messages on source queue moving to target queue
	# ------------------------------------------------------------------
	$MsgCnt = 0;
	while( $CompCode == 0 ) {
	    my $GetMsgDesc = {};
	    my $GetMsgOptions = MQGMO_WAIT | MQPMO_SYNCPOINT | MQGMO_FAIL_IF_QUIESCING;
	    my $GetMsgOpts = {
		    Options => $GetMsgOptions,
		    WaitInterval => 10000, ## in milliseconds.  10,000 is 10 seconds, etc.
		};
	    my $Buffer = MQGET($HConn, $HobjIP, $GetMsgDesc, $GetMsgOpts, $BufferLength, $CompCode, $Reason);
	    if ($CompCode != 0){
		if ( $Reason != 2033 ) {
		    print STDERR "Problem Getting from Queue: ",$ObjDescIP->{ObjectName}, "\n";
		}
	    } else {
		my $PutMsgOptsOptions = MQPMO_SYNCPOINT | MQOO_PASS_ALL_CONTEXT | MQPMO_FAIL_IF_QUIESCING;
		my $PutMsgOpts = {
		    Options => $PutMsgOptsOptions,
		    Context => $HobjIP,
		};
		my $Resp = MQPUT($HConn, $HobjOP, $GetMsgDesc, $PutMsgOpts, $Buffer, $CompCode, $Reason);
		if ($CompCode == 0){
		    MQCMIT($HConn, $CompCode, $Reason);
		    $MsgCnt++;
		} else {
		    my $SaveCC  = $CompCode;
		    my $SaveRsn = $Reason;
		    MQBACK($HConn, $CompCode, $Reason);
		    $CompCode = $SaveCC;
		    $Reason   = $SaveRsn;
		}
	    }
	}
	if ($Reason && $Reason != 2033) {
		print STDERR "$Reason: ", MQReasonToText($Reason), "\n";
	}
	print STDERR "\n";
    }
    # ------------------------------------------------------------------
    # Close queues
    # ------------------------------------------------------------------
    MQCLOSE($HConn,$HobjOP,MQCO_NONE,$CompCode,$Reason);
    MQCLOSE($HConn,$HobjIP,MQCO_NONE,$CompCode,$Reason);
}

# ------------------------------------------------------------------
# Disconnect.  Especially important for Clients.
# ------------------------------------------------------------------

my $Resp = MQDISC($HConn, $CompCode, $Reason);
if ($CompCode == 0){
    print STDERR "DisConnected from Queue Manager $QMgrName \n";}
else {
    print STDERR "Problem DisConnecting from Queue Manager: $QMgrName \n";
    print STDERR "$Reason: ", MQReasonToText($Reason), "\n";
}
print STDERR "$MsgCnt messages moved from $SourceQueue to $TargetQueue\n";
exit;
